# Vivy

## Prerequisites

- [rust](https://www.rust-lang.org) compiler with version >= 1.77
- [cmake](https://cmake.org/) at least the version 3.18
- C++ compiler with [C++17 support](https://en.cppreference.com/w/cpp/17)
- [mpv](https://mpv.io/) development library
- [Qt6](https://www.qt.io/) development library: QtCore, QtWidgets, QtOpenGL, QtOpenGLWidgets.
- The AV libraries in a recent version: libavutil libavcodec libavformat
- [cbindgen](https://github.com/mozilla/cbindgen): `cargo install --force cbindgen`
- [cargo-insta](https://insta.rs/docs/cli/): `cargo install --force cargo-insta`
- Some unix utils, the `jq` binary, the `bash` shell.

## Build

Simply use cmake to build in another folder of the source folder:

```bash
# Prepare, compile and install
cmake -Bbuild -DCMAKE_CXX_COMPILER=clang++ -DCMAKE_INSTALL_PREFIX=$HOME/.local -GNinja
ninja -Cbuild
ninja -Cbuild install
```

To do a debug build, use the debug switch with cmake: `-DCMAKE_BUILD_TYPE=Debug`.

To only build and use vvcc, you may use the following commands:

```bash
cd src/Rust

# Build and install, if vvcc is too big, you can use upx
cargo install --path vvs_cli

# Man pages
mkdir -p $HOME/.local/share/man/man1/
vvcc --manpage > $HOME/.local/share/man/man1/vvcc.1

# Shell completion scripts
mkdir -p $HOME/.local/share/completion/
vvcc --shell bash > $HOME/.local/share/bash-completion/completions/vvcc
vvcc --shell zsh  > $HOME/.local/share/zsh/site-functions/_vvcc

# View the dependencies
cargo install cargo-depgraph
cargo depgraph --build-deps --dedup-transitive-deps | dot -Tpdf | zathura -
```

## Licence

- The C++ part of this software is licenced under [the LGPL v3.0 or latter](/LICENSE)
- The most of the [Rust part](/src/Rust) (Vivy Script) is under the [MIT license](/src/Rust/LICENSE.txt)
- The NotoSans fonts are distributed using the [OFL licence](/rsc/licence/OFL-1.1)
- Some parts of the Rust folder ([vvs_parser](/src/Rust/vvs_parser), [vvs_parser_derive](/src/Rust/vvs_parser_derive))
  are under the [MPL 2.0](/src/Rust/full_moon/LICENSE.txt) and was originally written by *Kampfkarren <kampfkarren@gmail.com>*

