#include "PreCompiledHeaders.hh"
#include "Lib/Document//CRTPSubDocument.hh"
#include "Lib/JsonBuilder.hh"

using namespace Vivy;

// Init a ASS sub-document from a file
void
AssSubDocument::initFromPath(const QString &path)
{
    styles.clear();
    lines.clear();
    assFactory = new Ass::AssFactory(path);
    assFactory->getStyles(styles);
    assFactory->getLines(lines);
}

void
AssSubDocument::initFromInternal(const QJsonValue &internal)
{
    internalAss = true;
    styles.clear();
    lines.clear();
    assFactory = new Ass::AssFactory(internal);
    assFactory->getStyles(styles);
    assFactory->getLines(lines);
}

QString
AssSubDocument::getElementName() const noexcept
{
    return "AssSubDocument";
}

const QVector<Ass::LinePtr> &
AssSubDocument::getLines() const noexcept
{
    return lines;
}

const QVector<Ass::StylePtr> &
AssSubDocument::getStyles() const noexcept
{
    return styles;
}

QJsonDocument
AssSubDocument::getProperties() const noexcept
{
    QJsonObject styleObject;
    for (const Ass::StylePtr &style : styles) {
        styleObject.insert(style->getElementName(), style->getProperties().object());
    }

    return JsonBuilder::createOrderedJsonDocument({
        { "File", filePath },
        { "Styles", styleObject },
    });
}

Ass::AssFactory *
AssSubDocument::getFactory() noexcept
{
    return assFactory;
}

bool
AssSubDocument::haveInternalAss() const noexcept
{
    return internalAss;
}

void
AssSubDocument::setInternalAss(bool b) noexcept
{
    internalAss = b;
}

QJsonArray
AssSubDocument::getInternalAss() noexcept
{
    QJsonArray ret;

    QStringList info = assFactory->getAssHeaderAsText();
    for (int i = 0; i < info.size(); ++i)
        ret.append(info.at(i));

    for (int i = 0; i < lines.size(); i++) {
        ret.append(lines[i]->getLineAsText());
    }

    return ret;
}

void
AssSubDocument::setFilePath(QString filepath) noexcept
{
    filePath = filepath;
}
