use crate::prelude::*;
use llvm_sys::{core::*, prelude::*};

crate::bindings::declare! { const LLVMValueRef as Value<'a> }
crate::bindings::declare! { mut   LLVMValueRef as ValueMut<'a> }

impl Value<'_> {
    /// Get the type of the value.
    pub fn ty(&self) -> Type {
        unsafe { Type::from_ptr(LLVMTypeOf(self.as_ptr())) }
    }

    /// Create a clone of the value. Because we don't follow the semantic of rust for LLVM values,
    /// we won't call clone to not be confusing.
    ///
    /// If the cloned instruction is not inserted, you must call [Value::delete]
    pub fn create_clone(&self) -> Self {
        unsafe { Self::from_ptr(LLVMInstructionClone(self.as_ptr())) }
    }

    /// Delete an instruction.
    ///
    /// # Safety
    /// The instruction must not be inserted into any basic block.
    pub unsafe fn delete(self) {
        unsafe { LLVMDeleteInstruction(self.as_ptr()) }
    }
}

impl<'a> ValueMut<'a> {
    /// Get a const reference to the internal value out of the mutable value.
    pub fn into_ref(self) -> Value<'a> {
        unsafe { Value::from_ptr(self.as_ptr()) }
    }
}

impl Eq for Value<'_> {}
impl PartialEq for Value<'_> {
    fn eq(&self, other: &Self) -> bool {
        self.inner == other.inner
    }
}

impl Eq for ValueMut<'_> {}
impl PartialEq for ValueMut<'_> {
    fn eq(&self, other: &Self) -> bool {
        self.inner == other.inner
    }
}

impl<'a> AsRef<Value<'a>> for ValueMut<'a> {
    fn as_ref(&self) -> &Value<'a> {
        todo!()
    }
}
