use crate::types::*;
use std::{ptr::NonNull, rc::Rc};
use vvs_ass::ASSSyllabe;

/// Represent a syllabe, in an ffi-safe way. We represent if the syllabe was create by using an Rc
/// or was built from a reference.
#[derive(Debug, Clone, Copy)]
#[repr(C)]
pub struct VVRTSyllabe(NonNull<ASSSyllabe<VVRTTable>>);
crate::types::any::impl_try_from_any! { ASSSyllabe => VVRTSyllabe }

impl VVRTSyllabe {
    #[inline]
    pub fn dangling() -> Self {
        Self(NonNull::dangling())
    }

    #[inline]
    pub fn as_ptr(self) -> *mut u64 {
        self.0.as_ptr() as *mut _
    }

    #[inline]
    pub fn is_rc(&self) -> bool {
        !self.as_ref().aux.rt_infos().is_ass_borrowed
    }
}

impl From<ASSSyllabe<VVRTTable>> for VVRTSyllabe {
    fn from(mut value: ASSSyllabe<VVRTTable>) -> Self {
        value.aux.rt_infos_mut().is_ass_borrowed = false;
        let ptr = Rc::into_raw(Rc::new(value)) as *mut _;
        VVRTSyllabe(unsafe { NonNull::new_unchecked(ptr) })
    }
}

impl AsRef<ASSSyllabe<VVRTTable>> for VVRTSyllabe {
    fn as_ref(&self) -> &ASSSyllabe<VVRTTable> {
        unsafe { self.0.as_ref() }
    }
}

impl AsMut<ASSSyllabe<VVRTTable>> for VVRTSyllabe {
    fn as_mut(&mut self) -> &mut ASSSyllabe<VVRTTable> {
        unsafe { self.0.as_mut() }
    }
}


impl Eq for VVRTSyllabe {}
impl PartialEq for VVRTSyllabe {
    fn eq(&self, other: &Self) -> bool {
        self.as_ref().eq(other.as_ref())
    }
}

/// Module used to defined functions to be exported to the JIT.
#[allow(clippy::missing_safety_doc)]
#[vvs_procmacro::vvrt_bridge(VVRTSyllabe)]
mod ffi {
    #[no_mangle]
    pub unsafe extern "C" fn deep_clone(this: VVRTSyllabe) -> VVRTSyllabe {
        this.as_ref().clone().into()
    }

    #[no_mangle]
    unsafe extern "C" fn clone(this @ VVRTSyllabe(ptr): VVRTSyllabe) -> VVRTSyllabe {
        match this.is_rc() {
            false => VVRTSyllabe_deep_clone(this),
            true => {
                Rc::increment_strong_count(ptr.as_ptr());
                this
            }
        }
    }

    #[no_mangle]
    unsafe extern "C" fn drop(this @ VVRTSyllabe(ptr): VVRTSyllabe) {
        if this.is_rc() {
            Rc::decrement_strong_count(ptr.as_ptr())
        }
    }

    #[no_mangle]
    unsafe extern "C" fn len(this: VVRTSyllabe) -> i32 {
        this.as_ref().content.len().clamp(0, i32::MAX as usize) as i32
    }

    #[rustfmt::skip] #[no_mangle] pub unsafe extern "C" fn eq(this: VVRTSyllabe, other: VVRTSyllabe) -> bool { PartialEq::eq(&this, &other) }
    #[rustfmt::skip] #[no_mangle] pub unsafe extern "C" fn ne(this: VVRTSyllabe, other: VVRTSyllabe) -> bool { PartialEq::ne(&this, &other) }
}
