#pragma once

#include "PreCompiledHeaders.hh"
#include "UI/Utils.hh"
#include "Lib/Log.hh"

namespace Vivy
{
class LogConsole;
class ConsoleLogSinkDispatcher;

class LogConsole final : public QListWidget {
    VIVY_UNMOVABLE_OBJECT(LogConsole)

    int messageLimit{ 1'000 };
    std::weak_ptr<ConsoleLogSinkDispatcher> dispatcher;

public:
    LogConsole(QWidget *parent = nullptr) noexcept;
    ~LogConsole() noexcept override;

    void setMessageCountLimit(int limit);
    void handleLogMessage(const std::string_view, const LogMessage &) noexcept;

    // Hacky boi to set the `LogConsole *` in the dispatcher to nullptr when
    // the `LogConsole` is destroyed.
    void setDispatcher(std::shared_ptr<ConsoleLogSinkDispatcher> ptr) noexcept;
};

// Log console dispatcher, this is a proxy for the LogConsole which is the real
// widget, we don't directly set the dispatcher being the widget because the
// QDockWidget will demete its child on destruction, but the dispatcher is
// already managed by the std::shared_ptr...
class ConsoleLogSinkDispatcher final
    : public LogSinkDispatcher,
      public std::enable_shared_from_this<ConsoleLogSinkDispatcher> {
    VIVY_UNMOVABLE_OBJECT(ConsoleLogSinkDispatcher)

    LogConsole *console{ nullptr };

public:
    explicit ConsoleLogSinkDispatcher() noexcept;
    void handleLogMessage(const std::string_view, const LogMessage &) noexcept override;

    void attachLogConsole(LogConsole *const) noexcept;
};
}
