#pragma once

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Log.hh"
#include "Lib/Document/VivyDocument.hh"
#include "UI/PropertyModel.hh"
#include "UI/UnclosableDockWidget.hh"
#include "UI/AbstractDocumentView.hh"
#include "UI/DocumentViews/VideoView.hh"
#include "UI/DocumentViews/AssLinesView.hh"
#include "UI/DocumentViews/AssLinesModel.hh"

namespace Vivy
{
class VivyDocumentView final : public AbstractDocumentView {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(VivyDocumentView)
    VIVY_APP_LOGGABLE_OBJECT(VivyDocumentView, logger)

    static inline constexpr QDockWidget::DockWidgetFeatures allDockFeatures =
        QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetFloatable |
        QDockWidget::DockWidgetMovable;
    static inline constexpr QDockWidget::DockWidgetFeatures onlyClosableDockFeatures =
        QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable;

public:
    explicit VivyDocumentView(std::shared_ptr<VivyDocument>, QWidget *parent) noexcept;
    ~VivyDocumentView() noexcept override;

    void closeDocument() noexcept override;
    void openProperties() noexcept override;

    QString getDocumentTabName() const noexcept override;
    QString getDocumentTabToolTip() const noexcept override;
    QIcon getDocumentTabIcon() const noexcept override;

    VivyDocument *getDocument() const noexcept override;

    void loadViews() noexcept override;

public slots:
    void loadAudioView() noexcept;
    void loadVideoView() noexcept;
    void loadAssView() noexcept;

private:
    std::shared_ptr<VivyDocument> document;
    std::unique_ptr<PropertyModel> propertyModel{ nullptr };
    std::unique_ptr<AssLinesModel> assModel{ nullptr };
    UnclosableDockWidget *visualizer{ nullptr };
    UnclosableDockWidget *property{ nullptr };
    UnclosableDockWidget *assLines{ nullptr };
    UnclosableDockWidget *videoView{ nullptr };
    QAction *openPropertiesAct{ nullptr };

public:
    AssLinesView *getAssLinesView() const noexcept
    {
        return assLines == nullptr ? nullptr : reinterpret_cast<AssLinesView *>(assLines->widget());
    }

    AssLinesModel *getAssLinesModel() const noexcept { return assModel.get(); }

signals:
    // FIXME: fix default value or just send void?
    void audioSubDocumentChanged(std::shared_ptr<AudioSubDocument> = nullptr);
    void videoSubDocumentChanged(std::shared_ptr<VideoSubDocument> = nullptr);
    void assSubDocumentChanged(std::shared_ptr<AssSubDocument> = nullptr);
};
}
